@@ -19,17 +19,20 @@ binding.setupBufferJS(Buffer.prototype, bindingObj);
 const flags = bindingObj.flags;
 const kNoZeroFill = 0;
 
-function createBuffer(size) {
-  const ui8 = new Uint8Array(size);
-  Object.setPrototypeOf(ui8, Buffer.prototype);
-  return ui8;
+function createBuffer(size, noZeroFill) {
+  flags[kNoZeroFill] = noZeroFill ? 1 : 0;
+  try {
+    const ui8 = new Uint8Array(size);
+    Object.setPrototypeOf(ui8, Buffer.prototype);
+    return ui8;
+  } finally {
+    flags[kNoZeroFill] = 0;
+  }
 }
 
 function createPool() {
   poolSize = Buffer.poolSize;
-  if (poolSize > 0)
-    flags[kNoZeroFill] = 1;
-  allocPool = createBuffer(poolSize);
+  allocPool = createBuffer(poolSize, true);
   poolOffset = 0;
 }
 createPool();
@@ -62,16 +65,75 @@ function Buffer(arg, encoding) {
   return fromObject(arg);
 }
 
+/**
+ * Functionally equivalent to Buffer(arg, encoding) but throws a TypeError
+ * if value is a number.
+ * Buffer.from(str[, encoding])
+ * Buffer.from(array)
+ * Buffer.from(buffer)
+ * Buffer.from(arrayBuffer)
+ **/
+Buffer.from = function(value, encoding) {
+  if (typeof value === 'number')
+    throw new TypeError('"value" argument must not be a number');
+
+  if (typeof value === 'string') {
+    return fromString(value, encoding);
+  }
+
+  // Unusual.
+  return fromObject(value);
+};
+
 Object.setPrototypeOf(Buffer.prototype, Uint8Array.prototype);
 Object.setPrototypeOf(Buffer, Uint8Array);
 
+/**
+ * Creates a new filled Buffer instance.
+ * alloc(size[, fill[, encoding]])
+ **/
+Buffer.alloc = function(size, fill, encoding) {
+  if (typeof size !== 'number')
+    throw new TypeError('"size" argument must be a number');
+  if (size <= 0)
+    return createBuffer(size);
+  if (fill !== undefined) {
+    // Only pay attention to encoding if it's a string. This
+    // prevents accidentally sending in a number that would
+    // be interpretted as a start offset.
+    // Also, don't apply encoding if fill is a number.
+    if (typeof fill !== 'number' && typeof encoding === 'string')
+      fill = Buffer.from(fill, encoding);
+
+    return createBuffer(size, true).fill(fill);
+  }
+  return createBuffer(size);
+};
+
+/**
+ * Equivalent to Buffer(num), by default creates a non-zero-filled Buffer
+ * instance.
+ **/
+Buffer.allocUnsafe = function(size) {
+  if (typeof size !== 'number')
+    throw new TypeError('"size" argument must be a number');
+  return allocate(size);
+};
+
+/**
+ * Equivalent to SlowBuffer(num), by default creates a non-zero-filled
+ * Buffer instance that is not allocated off the pre-initialized pool.
+ **/
+Buffer.allocUnsafeSlow = function(size) {
+  if (typeof size !== 'number')
+    throw new TypeError('"size" argument must be a number');
+  return createBuffer(size, true);
+};
 
 function SlowBuffer(length) {
   if (+length != length)
     length = 0;
-  if (length > 0)
-    flags[kNoZeroFill] = 1;
-  return createBuffer(+length);
+  return createBuffer(+length, true);
 }
 
 Object.setPrototypeOf(SlowBuffer.prototype, Uint8Array.prototype);
@@ -79,8 +141,8 @@ Object.setPrototypeOf(SlowBuffer, Uint8Array);
 
 
 function allocate(size) {
-  if (size === 0) {
-    return createBuffer(size);
+  if (size <= 0) {
+    return createBuffer(0);
   }
   if (size < (Buffer.poolSize >>> 1)) {
     if (size > (poolSize - poolOffset))
@@ -93,9 +155,7 @@ function allocate(size) {
     // Even though this is checked above, the conditional is a safety net and
     // sanity check to prevent any subsequent typed array allocation from not
     // being zero filled.
-    if (size > 0)
-      flags[kNoZeroFill] = 1;
-    return createBuffer(size);
+    return createBuffer(size, true);
   }
 }
 
@@ -117,10 +177,17 @@ function fromString(string, encoding) {
   return b;
 }
 
+function fromArrayLike(obj) {
+  const length = obj.length;
+  const b = allocate(length);
+  for (var i = 0; i < length; i++)
+    b[i] = obj[i] & 255;
+  return b;
+}
 
 function fromObject(obj) {
   if (obj instanceof Buffer) {
-    var b = allocate(obj.length);
+    const b = allocate(obj.length);
 
     if (b.length === 0)
       return b;
@@ -129,14 +196,6 @@ function fromObject(obj) {
     return b;
   }
 
-  if (Array.isArray(obj)) {
-    var length = obj.length;
-    var b = allocate(length);
-    for (var i = 0; i < length; i++)
-      b[i] = obj[i] & 255;
-    return b;
-  }
-
   if (obj == null) {
     throw new TypeError('must start with number, buffer, array or string');
   }
@@ -145,25 +204,15 @@ function fromObject(obj) {
     return binding.createFromArrayBuffer(obj);
   }
 
-  if (obj.buffer instanceof ArrayBuffer || obj.length) {
-    var length;
-    if (typeof obj.length !== 'number' || obj.length !== obj.length)
-      length = 0;
-    else
-      length = obj.length;
-    var b = allocate(length);
-    for (var i = 0; i < length; i++) {
-      b[i] = obj[i] & 255;
+  if (obj.buffer instanceof ArrayBuffer || 'length' in obj) {
+    if (typeof obj.length !== 'number' || obj.length !== obj.length) {
+      return allocate(0);
     }
-    return b;
+    return fromArrayLike(obj);
   }
 
   if (obj.type === 'Buffer' && Array.isArray(obj.data)) {
-    var array = obj.data;
-    var b = allocate(array.length);
-    for (var i = 0; i < array.length; i++)
-      b[i] = array[i] & 255;
-    return b;
+    return fromArrayLike(obj.data);
   }
 
   throw new TypeError('must start with number, buffer, array or string');
@@ -219,6 +268,7 @@ Buffer.isEncoding = function(encoding) {
 
 
 Buffer.concat = function(list, length) {
+  var i;
   if (!Array.isArray(list))
     throw new TypeError('list argument must be an Array of Buffers.');
 
@@ -227,7 +277,7 @@ Buffer.concat = function(list, length) {
 
   if (length === undefined) {
     length = 0;
-    for (var i = 0; i < list.length; i++)
+    for (i = 0; i < list.length; i++)
       length += list[i].length;
   } else {
     length = length >>> 0;
@@ -235,12 +285,20 @@ Buffer.concat = function(list, length) {
 
   var buffer = new Buffer(length);
   var pos = 0;
-  for (var i = 0; i < list.length; i++) {
+  for (i = 0; i < list.length; i++) {
     var buf = list[i];
     buf.copy(buffer, pos);
     pos += buf.length;
   }
 
+  // Note: `length` is always equal to `buffer.length` at this point
+  if (pos < length) {
+    // Zero-fill the remaining bytes if the specified `length` was more than
+    // the actual total length, i.e. if we have some remaining allocated bytes
+    // there were not initialized.
+    buffer.fill(0, pos, length);
+  }
+
   return buffer;
 };
 
@@ -258,6 +316,9 @@ function base64ByteLength(str, bytes) {
 
 
 function byteLength(string, encoding) {
+  if (string instanceof Buffer)
+    return string.length;
+
   if (typeof string !== 'string')
     string = '' + string;
 
@@ -272,6 +333,7 @@ function byteLength(string, encoding) {
       case 'ascii':
       case 'binary':
       // Deprecated
+      // eslint-disable-next-line no-fallthrough
       case 'raw':
       case 'raws':
         return len;
@@ -376,10 +438,11 @@ function slowToString(encoding, start, end) {
 
 
 Buffer.prototype.toString = function() {
+  let result;
   if (arguments.length === 0) {
-    var result = this.utf8Slice(0, this.length);
+    result = this.utf8Slice(0, this.length);
   } else {
-    var result = slowToString.apply(this, arguments);
+    result = slowToString.apply(this, arguments);
   }
   if (result === undefined)
     throw new Error('toString failed');
@@ -438,7 +501,7 @@ function slowIndexOf(buffer, val, byteOffset, encoding) {
       case 'ascii':
       case 'hex':
         return binding.indexOfBuffer(
-            buffer, Buffer(val, encoding), byteOffset, encoding);
+            buffer, Buffer.from(val, encoding), byteOffset, encoding);
 
       default:
         if (loweredCase) {
@@ -452,10 +515,14 @@ function slowIndexOf(buffer, val, byteOffset, encoding) {
 }
 
 Buffer.prototype.indexOf = function indexOf(val, byteOffset, encoding) {
-  if (byteOffset > 0x7fffffff)
+  if (typeof byteOffset === 'string') {
+    encoding = byteOffset;
+    byteOffset = 0;
+  } else if (byteOffset > 0x7fffffff) {
     byteOffset = 0x7fffffff;
-  else if (byteOffset < -0x80000000)
+  } else if (byteOffset < -0x80000000) {
     byteOffset = -0x80000000;
+  }
   byteOffset >>= 0;
 
   if (typeof val === 'string') {
@@ -488,6 +555,10 @@ Buffer.prototype.fill = function fill(val, start, end) {
     var code = val.charCodeAt(0);
     if (code < 256)
       val = code;
+  } else if (val.length === 0) {
+    // Previously, if val === '', the Buffer would not fill,
+    // which is rather surprising.
+    val = 0;
   }
 
   binding.fill(this, val, start, end);
@@ -513,9 +584,6 @@ Buffer.prototype.set = internalUtil.deprecate(function set(offset, v) {
   return this[offset] = v;
 }, 'Buffer.set is deprecated. Use array indexes instead.');
 
-
-// TODO(trevnorris): fix these checks to follow new standard
-// write(string, offset = 0, length = buffer.length, encoding = 'utf8')
 var writeWarned = false;
 const writeMsg = 'Buffer.write(string, encoding, offset, length) is ' +
                  'deprecated. Use write(string[, offset[, length]]' +
@@ -608,8 +676,6 @@ Buffer.prototype.toJSON = function() {
 };
 
 
-// TODO(trevnorris): currently works like Array.prototype.slice(), which
-// doesn't follow the new standard for throwing on out of range indexes.
 Buffer.prototype.slice = function slice(start, end) {
   const buffer = this.subarray(start, end);
   Object.setPrototypeOf(buffer, Buffer.prototype);
