@@ -33,8 +33,12 @@ function Worker(options) {
 
   if (options.process) {
     this.process = options.process;
-    this.process.on('error', this.emit.bind(this, 'error'));
-    this.process.on('message', this.emit.bind(this, 'message'));
+    this.process.on('error', (code, signal) =>
+      this.emit('error', code, signal)
+    );
+    this.process.on('message', (message, handle) =>
+      this.emit('message', message, handle)
+    );
   }
 }
 util.inherits(Worker, EventEmitter);
@@ -233,9 +237,8 @@ function masterInit() {
     // Without --logfile=v8-%p.log, everything ends up in a single, unusable
     // file. (Unusable because what V8 logs are memory addresses and each
     // process has its own memory mappings.)
-    if (settings.execArgv.some(function(s) { return /^--prof/.test(s); }) &&
-        !settings.execArgv.some(function(s) { return /^--logfile=/.test(s); }))
-    {
+    if (settings.execArgv.some((s) => s.startsWith('--prof')) &&
+        !settings.execArgv.some((s) => s.startsWith('--logfile='))) {
       settings.execArgv = settings.execArgv.concat(['--logfile=v8-%p.log']);
     }
     cluster.settings = settings;
@@ -283,6 +286,7 @@ function masterInit() {
   function createWorkerProcess(id, env) {
     var workerEnv = util._extend({}, process.env);
     var execArgv = cluster.settings.execArgv.slice();
+    var debugPort = 0;
 
     workerEnv = util._extend(workerEnv, env);
     workerEnv.NODE_UNIQUE_ID = '' + id;
@@ -291,8 +295,11 @@ function masterInit() {
       var match = execArgv[i].match(/^(--debug|--debug-(brk|port))(=\d+)?$/);
 
       if (match) {
-        const debugPort = process.debugPort + debugPortOffset;
-        ++debugPortOffset;
+        if (debugPort === 0) {
+          debugPort = process.debugPort + debugPortOffset;
+          ++debugPortOffset;
+        }
+
         execArgv[i] = match[1] + '=' + debugPort;
       }
     }
@@ -337,7 +344,9 @@ function masterInit() {
       process: workerProcess
     });
 
-    worker.on('message', this.emit.bind(this, 'message'));
+    worker.on('message', (message, handle) =>
+      cluster.emit('message', message, handle)
+    );
 
     worker.process.once('exit', function(exitCode, signalCode) {
       /*
@@ -428,7 +437,7 @@ function masterInit() {
     else if (message.act === 'listening')
       listening(worker, message);
     else if (message.act === 'suicide')
-      worker.suicide = true;
+      suicide(worker, message);
     else if (message.act === 'close')
       close(worker, message);
   }
@@ -439,7 +448,15 @@ function masterInit() {
     cluster.emit('online', worker);
   }
 
+  function suicide(worker, message) {
+    worker.suicide = true;
+    send(worker, { ack: message.seq });
+  }
+
   function queryServer(worker, message) {
+    // Stop processing if worker already disconnecting
+    if (worker.suicide)
+      return;
     var args = [message.address,
                 message.port,
                 message.addressType,
@@ -532,24 +549,24 @@ function workerInit() {
       if (message.act === 'newconn')
         onconnection(message, handle);
       else if (message.act === 'disconnect')
-        worker.disconnect();
+        _disconnect.call(worker, true);
     }
   };
 
   // obj is a net#Server or a dgram#Socket object.
   cluster._getServer = function(obj, options, cb) {
-    const key = [ options.address,
-                options.port,
-                options.addressType,
-                options.fd ].join(':');
-    if (indexes[key] === undefined)
-      indexes[key] = 0;
+    const indexesKey = [ options.address,
+                         options.port,
+                         options.addressType,
+                         options.fd ].join(':');
+    if (indexes[indexesKey] === undefined)
+      indexes[indexesKey] = 0;
     else
-      indexes[key]++;
+      indexes[indexesKey]++;
 
     const message = util._extend({
       act: 'queryServer',
-      index: indexes[key],
+      index: indexes[indexesKey],
       data: null
     }, options);
 
@@ -559,9 +576,9 @@ function workerInit() {
       if (obj._setServerData) obj._setServerData(reply.data);
 
       if (handle)
-        shared(reply, handle, cb);  // Shared listen socket.
+        shared(reply, handle, indexesKey, cb);  // Shared listen socket.
       else
-        rr(reply, cb);              // Round-robin.
+        rr(reply, indexesKey, cb);              // Round-robin.
     });
     obj.once('listening', function() {
       cluster.worker.state = 'listening';
@@ -573,7 +590,7 @@ function workerInit() {
   };
 
   // Shared listen socket.
-  function shared(message, handle, cb) {
+  function shared(message, handle, indexesKey, cb) {
     var key = message.key;
     // Monkey-patch the close() method so we can keep track of when it's
     // closed. Avoids resource leaks when the handle is short-lived.
@@ -581,6 +598,7 @@ function workerInit() {
     handle.close = function() {
       send({ act: 'close', key: key });
       delete handles[key];
+      delete indexes[indexesKey];
       return close.apply(this, arguments);
     };
     assert(handles[key] === undefined);
@@ -589,7 +607,7 @@ function workerInit() {
   }
 
   // Round-robin. Master distributes handles across workers.
-  function rr(message, cb) {
+  function rr(message, indexesKey, cb) {
     if (message.errno)
       return cb(message.errno, null);
 
@@ -610,6 +628,7 @@ function workerInit() {
       if (key === undefined) return;
       send({ act: 'close', key: key });
       delete handles[key];
+      delete indexes[indexesKey];
       key = undefined;
     }
 
@@ -653,14 +672,36 @@ function workerInit() {
   }
 
   Worker.prototype.disconnect = function() {
+    _disconnect.call(this);
+  };
+
+  Worker.prototype.destroy = function() {
+    this.suicide = true;
+    if (!this.isConnected()) process.exit(0);
+    var exit = process.exit.bind(null, 0);
+    send({ act: 'suicide' }, () => process.disconnect());
+    process.once('disconnect', exit);
+  };
+
+  function send(message, cb) {
+    sendHelper(process, message, null, cb);
+  }
+
+  function _disconnect(masterInitiated) {
     this.suicide = true;
     let waitingCount = 1;
 
     function checkWaitingCount() {
       waitingCount--;
       if (waitingCount === 0) {
-        send({ act: 'suicide' });
-        process.disconnect();
+        // If disconnect is worker initiated, wait for ack to be sure suicide
+        // is properly set in the master, otherwise, if it's master initiated
+        // there's no need to send the suicide message
+        if (masterInitiated) {
+          process.disconnect();
+        } else {
+          send({ act: 'suicide' }, () => process.disconnect());
+        }
       }
     }
 
@@ -668,23 +709,14 @@ function workerInit() {
       const handle = handles[key];
       delete handles[key];
       waitingCount++;
-      handle.owner.close(checkWaitingCount);
+
+      if (handle.owner)
+        handle.owner.close(checkWaitingCount);
+      else
+        handle.close(checkWaitingCount);
     }
 
     checkWaitingCount();
-  };
-
-  Worker.prototype.destroy = function() {
-    this.suicide = true;
-    if (!this.isConnected()) process.exit(0);
-    var exit = process.exit.bind(null, 0);
-    send({ act: 'suicide' }, exit);
-    process.once('disconnect', exit);
-    process.disconnect();
-  };
-
-  function send(message, cb) {
-    sendHelper(process, message, null, cb);
   }
 }
 
@@ -692,6 +724,9 @@ function workerInit() {
 var seq = 0;
 var callbacks = {};
 function sendHelper(proc, message, handle, cb) {
+  if (!proc.connected)
+    return false;
+
   // Mark message as internal. See INTERNAL_PREFIX in lib/child_process.js
   message = util._extend({ cmd: 'NODE_CLUSTER' }, message);
   if (cb) callbacks[seq] = cb;
@@ -707,7 +742,7 @@ function internal(worker, cb) {
   return function(message, handle) {
     if (message.cmd !== 'NODE_CLUSTER') return;
     var fn = cb;
-    if (message.ack !== undefined) {
+    if (message.ack !== undefined && callbacks[message.ack] !== undefined) {
       fn = callbacks[message.ack];
       delete callbacks[message.ack];
     }
