@@ -5,6 +5,7 @@ const stream = require('stream');
 const timers = require('timers');
 const util = require('util');
 const internalUtil = require('internal/util');
+const internalNet = require('internal/net');
 const assert = require('assert');
 const cares = process.binding('cares_wrap');
 const uv = process.binding('uv');
@@ -22,6 +23,7 @@ const WriteWrap = process.binding('stream_wrap').WriteWrap;
 var cluster;
 const errnoException = util._errnoException;
 const exceptionWithHostPort = util._exceptionWithHostPort;
+const isLegalPort = internalNet.isLegalPort;
 
 function noop() {}
 
@@ -57,7 +59,11 @@ exports.createServer = function(options, connectionListener) {
 // connect(path, [cb]);
 //
 exports.connect = exports.createConnection = function() {
-  var args = normalizeConnectArgs(arguments);
+  const argsLen = arguments.length;
+  var args = new Array(argsLen);
+  for (var i = 0; i < argsLen; i++)
+    args[i] = arguments[i];
+  args = normalizeConnectArgs(args);
   debug('createConnection', args);
   var s = new Socket(args[0]);
   return Socket.prototype.connect.apply(s, args);
@@ -91,7 +97,6 @@ exports._normalizeConnectArgs = normalizeConnectArgs;
 // called when creating new Socket, or when re-using a closed Socket
 function initSocketHandle(self) {
   self.destroyed = false;
-  self.bytesRead = 0;
   self._bytesDispatched = 0;
   self._sockname = null;
 
@@ -106,6 +111,10 @@ function initSocketHandle(self) {
   }
 }
 
+
+const BYTES_READ = Symbol('bytesRead');
+
+
 function Socket(options) {
   if (!(this instanceof Socket)) return new Socket(options);
 
@@ -169,6 +178,13 @@ function Socket(options) {
       this.read(0);
     }
   }
+
+  // Reserve properties
+  this.server = null;
+  this._server = null;
+
+  // Used after `.destroy()`
+  this[BYTES_READ] = 0;
 }
 util.inherits(Socket, stream.Duplex);
 
@@ -267,9 +283,8 @@ function writeAfterFIN(chunk, encoding, cb) {
 
   var er = new Error('This socket has been ended by the other party');
   er.code = 'EPIPE';
-  var self = this;
   // TODO: defer error events consistently everywhere, not just the cb
-  self.emit('error', er);
+  this.emit('error', er);
   if (typeof cb === 'function') {
     process.nextTick(cb, er);
   }
@@ -290,9 +305,8 @@ Socket.prototype.read = function(n) {
 
 Socket.prototype.listen = function() {
   debug('socket.listen');
-  var self = this;
-  self.on('connection', arguments[0]);
-  listen(self, null, null, null);
+  this.on('connection', arguments[0]);
+  listen(this, null, null, null);
 };
 
 
@@ -322,7 +336,7 @@ Socket.prototype._onTimeout = function() {
 Socket.prototype.setNoDelay = function(enable) {
   if (!this._handle) {
     this.once('connect',
-              enable ? this.setNoDelay : this.setNoDelay.bind(this, enable));
+              enable ? this.setNoDelay : () => this.setNoDelay(enable));
     return this;
   }
 
@@ -336,7 +350,7 @@ Socket.prototype.setNoDelay = function(enable) {
 
 Socket.prototype.setKeepAlive = function(setting, msecs) {
   if (!this._handle) {
-    this.once('connect', this.setKeepAlive.bind(this, setting, msecs));
+    this.once('connect', () => this.setKeepAlive(setting, msecs));
     return this;
   }
 
@@ -384,7 +398,7 @@ Socket.prototype._read = function(n) {
 
   if (this._connecting || !this._handle) {
     debug('_read wait for connection');
-    this.once('connect', this._read.bind(this, n));
+    this.once('connect', () => this._read(n));
   } else if (!this._handle.reading) {
     // not already reading, start the flow
     debug('Socket._read readStart');
@@ -464,6 +478,9 @@ Socket.prototype._destroy = function(exception, cb) {
     if (this !== process.stderr)
       debug('close handle');
     var isException = exception ? true : false;
+    // `bytesRead` should be accessible after `.destroy()`
+    this[BYTES_READ] = this._handle.bytesRead;
+
     this._handle.close(function() {
       debug('emit close');
       self.emit('close', isException);
@@ -479,12 +496,12 @@ Socket.prototype._destroy = function(exception, cb) {
   this.destroyed = true;
   fireErrorCallbacks();
 
-  if (this.server) {
+  if (this._server) {
     COUNTER_NET_SERVER_CONNECTION_CLOSE(this);
     debug('has server');
-    this.server._connections--;
-    if (this.server._emitCloseIfDrained) {
-      this.server._emitCloseIfDrained();
+    this._server._connections--;
+    if (this._server._emitCloseIfDrained) {
+      this._server._emitCloseIfDrained();
     }
   }
 };
@@ -515,10 +532,6 @@ function onread(nread, buffer) {
     // will prevent this from being called again until _read() gets
     // called again.
 
-    // if it's not enough data, we'll just call handle.readStart()
-    // again right away.
-    self.bytesRead += nread;
-
     // Optimization: emit the original buffer with end points
     var ret = self.push(buffer);
 
@@ -574,16 +587,27 @@ Socket.prototype._getpeername = function() {
   return this._peername;
 };
 
+function protoGetter(name, callback) {
+  Object.defineProperty(Socket.prototype, name, {
+    configurable: false,
+    enumerable: true,
+    get: callback
+  });
+}
 
-Socket.prototype.__defineGetter__('remoteAddress', function() {
+protoGetter('bytesRead', function bytesRead() {
+  return this._handle ? this._handle.bytesRead : this[BYTES_READ];
+});
+
+protoGetter('remoteAddress', function remoteAddress() {
   return this._getpeername().address;
 });
 
-Socket.prototype.__defineGetter__('remoteFamily', function() {
+protoGetter('remoteFamily', function remoteFamily() {
   return this._getpeername().family;
 });
 
-Socket.prototype.__defineGetter__('remotePort', function() {
+protoGetter('remotePort', function remotePort() {
   return this._getpeername().port;
 });
 
@@ -602,12 +626,12 @@ Socket.prototype._getsockname = function() {
 };
 
 
-Socket.prototype.__defineGetter__('localAddress', function() {
+protoGetter('localAddress', function localAddress() {
   return this._getsockname().address;
 });
 
 
-Socket.prototype.__defineGetter__('localPort', function() {
+protoGetter('localPort', function localPort() {
   return this._getsockname().port;
 });
 
@@ -651,10 +675,8 @@ Socket.prototype._writeGeneric = function(writev, data, encoding, cb) {
     var chunks = new Array(data.length << 1);
     for (var i = 0; i < data.length; i++) {
       var entry = data[i];
-      var chunk = entry.chunk;
-      var enc = entry.encoding;
-      chunks[i * 2] = chunk;
-      chunks[i * 2 + 1] = enc;
+      chunks[i * 2] = entry.chunk;
+      chunks[i * 2 + 1] = entry.encoding;
     }
     err = this._handle.writev(req, chunks);
 
@@ -721,11 +743,11 @@ function createWriteReq(req, handle, data, encoding) {
 }
 
 
-Socket.prototype.__defineGetter__('bytesWritten', function() {
-  var bytes = this._bytesDispatched,
-      state = this._writableState,
-      data = this._pendingData,
-      encoding = this._pendingEncoding;
+protoGetter('bytesWritten', function bytesWritten() {
+  var bytes = this._bytesDispatched;
+  const state = this._writableState;
+  const data = this._pendingData;
+  const encoding = this._pendingEncoding;
 
   if (!state)
     return undefined;
@@ -760,7 +782,7 @@ function afterWrite(status, handle, req, err) {
   }
 
   if (status < 0) {
-    var ex = exceptionWithHostPort(status, 'write', req.address, req.port);
+    var ex = errnoException(status, 'write', req.error);
     debug('write failure', ex);
     self._destroy(ex, req.cb);
     return;
@@ -806,14 +828,14 @@ function connect(self, address, port, addressType, localAddress, localPort) {
     err = bind(localAddress, localPort);
 
     if (err) {
-      var ex = exceptionWithHostPort(err, 'bind', localAddress, localPort);
+      const ex = exceptionWithHostPort(err, 'bind', localAddress, localPort);
       self._destroy(ex);
       return;
     }
   }
 
   if (addressType === 6 || addressType === 4) {
-    var req = new TCPConnectWrap();
+    const req = new TCPConnectWrap();
     req.oncomplete = afterConnect;
     req.address = address;
     req.port = port;
@@ -826,7 +848,7 @@ function connect(self, address, port, addressType, localAddress, localPort) {
       err = self._handle.connect6(req, address, port);
 
   } else {
-    var req = new PipeConnectWrap();
+    const req = new PipeConnectWrap();
     req.address = address;
     req.oncomplete = afterConnect;
     err = self._handle.connect(req, address, afterConnect);
@@ -840,21 +862,12 @@ function connect(self, address, port, addressType, localAddress, localPort) {
       details = sockname.address + ':' + sockname.port;
     }
 
-    var ex = exceptionWithHostPort(err, 'connect', address, port, details);
+    const ex = exceptionWithHostPort(err, 'connect', address, port, details);
     self._destroy(ex);
   }
 }
 
 
-// Check that the port number is not NaN when coerced to a number,
-// is an integer and that it falls within the legal range of port numbers.
-function isLegalPort(port) {
-  if (typeof port === 'string' && port.trim() === '')
-    return false;
-  return +port === (port >>> 0) && port >= 0 && port <= 0xFFFF;
-}
-
-
 Socket.prototype.connect = function(options, cb) {
   if (this.write !== Socket.prototype.write)
     this.write = Socket.prototype.write;
@@ -863,7 +876,11 @@ Socket.prototype.connect = function(options, cb) {
     // Old API:
     // connect(port, [host], [cb])
     // connect(path, [cb]);
-    var args = normalizeConnectArgs(arguments);
+    const argsLen = arguments.length;
+    var args = new Array(argsLen);
+    for (var i = 0; i < argsLen; i++)
+      args[i] = arguments[i];
+    args = normalizeConnectArgs(args);
     return Socket.prototype.connect.apply(this, args);
   }
 
@@ -881,7 +898,6 @@ Socket.prototype.connect = function(options, cb) {
     this._sockname = null;
   }
 
-  var self = this;
   var pipe = !!options.path;
   debug('pipe', pipe, options.path);
 
@@ -891,21 +907,20 @@ Socket.prototype.connect = function(options, cb) {
   }
 
   if (typeof cb === 'function') {
-    self.once('connect', cb);
+    this.once('connect', cb);
   }
 
   this._unrefTimer();
 
-  self._connecting = true;
-  self.writable = true;
+  this._connecting = true;
+  this.writable = true;
 
   if (pipe) {
-    connect(self, options.path);
-
+    connect(this, options.path);
   } else {
-    lookupAndConnect(self, options);
+    lookupAndConnect(this, options);
   }
-  return self;
+  return this;
 };
 
 
@@ -931,7 +946,6 @@ function lookupAndConnect(self, options) {
   port |= 0;
 
   // If host is an IP, skip performing a lookup
-  // TODO(evanlucas) should we hot path this for localhost?
   var addressType = exports.isIP(host);
   if (addressType) {
     process.nextTick(function() {
@@ -951,12 +965,12 @@ function lookupAndConnect(self, options) {
 
   if (dnsopts.family !== 4 && dnsopts.family !== 6) {
     dnsopts.hints = dns.ADDRCONFIG;
-    // The AI_V4MAPPED hint is not supported on FreeBSD, and getaddrinfo
-    // returns EAI_BADFLAGS. However, it seems to be supported on most other
-    // systems. See
+    // The AI_V4MAPPED hint is not supported on FreeBSD or Android,
+    // and getaddrinfo returns EAI_BADFLAGS. However, it seems to be
+    // supported on most other systems. See
     // http://lists.freebsd.org/pipermail/freebsd-bugs/2008-February/028260.html
     // for more information on the lack of support for FreeBSD.
-    if (process.platform !== 'freebsd')
+    if (process.platform !== 'freebsd' && process.platform !== 'android')
       dnsopts.hints |= dns.V4MAPPED;
   }
 
@@ -1132,8 +1146,7 @@ function _listen(handle, backlog) {
   return handle.listen(backlog || 511);
 }
 
-var createServerHandle = exports._createServerHandle =
-    function(address, port, addressType, fd) {
+function createServerHandle(address, port, addressType, fd) {
   var err = 0;
   // assign handle in listen, and clean up if bind or listen fails
   var handle;
@@ -1188,16 +1201,16 @@ var createServerHandle = exports._createServerHandle =
   }
 
   return handle;
-};
+}
+exports._createServerHandle = createServerHandle;
 
 
 Server.prototype._listen2 = function(address, port, addressType, backlog, fd) {
   debug('listen2', address, port, addressType, backlog, fd);
-  var self = this;
 
   // If there is not yet a handle, we need to create one and bind.
   // In the case of a server sent via IPC, we don't need to do this.
-  if (self._handle) {
+  if (this._handle) {
     debug('_listen2: have a handle already');
   } else {
     debug('_listen2: create a handle');
@@ -1222,22 +1235,22 @@ Server.prototype._listen2 = function(address, port, addressType, backlog, fd) {
 
     if (typeof rval === 'number') {
       var error = exceptionWithHostPort(rval, 'listen', address, port);
-      process.nextTick(emitErrorNT, self, error);
+      process.nextTick(emitErrorNT, this, error);
       return;
     }
-    self._handle = rval;
+    this._handle = rval;
   }
 
-  self._handle.onconnection = onconnection;
-  self._handle.owner = self;
+  this._handle.onconnection = onconnection;
+  this._handle.owner = this;
 
-  var err = _listen(self._handle, backlog);
+  var err = _listen(this._handle, backlog);
 
   if (err) {
     var ex = exceptionWithHostPort(err, 'listen', address, port);
-    self._handle.close();
-    self._handle = null;
-    process.nextTick(emitErrorNT, self, ex);
+    this._handle.close();
+    this._handle = null;
+    process.nextTick(emitErrorNT, this, ex);
     return;
   }
 
@@ -1248,7 +1261,7 @@ Server.prototype._listen2 = function(address, port, addressType, backlog, fd) {
   if (this._unref)
     this.unref();
 
-  process.nextTick(emitListeningNT, self);
+  process.nextTick(emitListeningNT, this);
 };
 
 
@@ -1351,7 +1364,7 @@ Server.prototype.listen = function() {
         else
           listen(self, null, h.port | 0, 4, backlog, undefined, h.exclusive);
       } else if (h.path && isPipeName(h.path)) {
-        var pipeName = self._pipeName = h.path;
+        const pipeName = self._pipeName = h.path;
         listen(self, pipeName, -1, -1, backlog, undefined, h.exclusive);
       } else {
         throw new Error('Invalid listen argument: ' + h);
@@ -1359,7 +1372,7 @@ Server.prototype.listen = function() {
     }
   } else if (isPipeName(arguments[0])) {
     // UNIX socket or Windows pipe.
-    var pipeName = self._pipeName = arguments[0];
+    const pipeName = self._pipeName = arguments[0];
     listen(self, pipeName, -1, -1, backlog);
 
   } else if (arguments[1] === undefined ||
@@ -1426,6 +1439,7 @@ function onconnection(err, clientHandle) {
 
   self._connections++;
   socket.server = self;
+  socket._server = self;
 
   DTRACE_NET_SERVER_CONNECTION(socket);
   LTTNG_NET_SERVER_CONNECTION(socket);
@@ -1444,8 +1458,8 @@ Server.prototype.getConnections = function(cb) {
   }
 
   // Poll slaves
-  var left = this._slaves.length,
-      total = this._connections;
+  var left = this._slaves.length;
+  var total = this._connections;
 
   function oncount(err, count) {
     if (err) {
@@ -1487,8 +1501,8 @@ Server.prototype.close = function(cb) {
   }
 
   if (this._usingSlaves) {
-    var self = this,
-        left = this._slaves.length;
+    var self = this;
+    var left = this._slaves.length;
 
     // Increment connections to be sure that, even if all sockets will be closed
     // during polling of slaves, `close` event will be emitted only once.
@@ -1507,15 +1521,14 @@ Server.prototype.close = function(cb) {
 
 Server.prototype._emitCloseIfDrained = function() {
   debug('SERVER _emitCloseIfDrained');
-  var self = this;
 
-  if (self._handle || self._connections) {
+  if (this._handle || this._connections) {
     debug('SERVER handle? %j   connections? %d',
-          !!self._handle, self._connections);
+          !!this._handle, this._connections);
     return;
   }
 
-  process.nextTick(emitCloseNT, self);
+  process.nextTick(emitCloseNT, this);
 };
 
 
@@ -1557,12 +1570,12 @@ exports.isIP = cares.isIP;
 
 
 exports.isIPv4 = function(input) {
-  return exports.isIP(input) === 4;
+  return cares.isIPv4(input);
 };
 
 
 exports.isIPv6 = function(input) {
-  return exports.isIP(input) === 6;
+  return cares.isIPv6(input);
 };
 
 
