@@ -1,5 +1,5 @@
 'use strict';
-var common = require('../common');
+require('../common');
 var assert = require('assert');
 var http = require('http');
 var net = require('net');
@@ -8,7 +8,7 @@ var caughtError = false;
 
 var options = {
   host: '127.0.0.1',
-  port: common.PORT
+  port: undefined
 };
 
 // start a tcp server that closes incoming connections immediately
@@ -16,10 +16,11 @@ var server = net.createServer(function(client) {
   client.destroy();
   server.close();
 });
-server.listen(options.port, options.host, onListen);
+server.listen(0, options.host, onListen);
 
 // do a GET request, expect it to fail
 function onListen() {
+  options.port = this.address().port;
   var req = http.request(options, function(res) {
     assert.ok(false, 'this should never run');
   });
