@@ -1,5 +1,5 @@
 'use strict';
-var common = require('../common');
+require('../common');
 var assert = require('assert');
 var net = require('net');
 
@@ -7,8 +7,8 @@ var received = '';
 
 var server = net.createServer(function(socket) {
   socket.pipe(socket);
-}).listen(common.PORT, function() {
-  var conn = net.connect(common.PORT);
+}).listen(0, function() {
+  var conn = net.connect(this.address().port);
   conn.setEncoding('utf8');
   conn.write('before');
   conn.on('connect', function() {
