@@ -4,4 +4,4 @@ var assert = require('assert');
 
 var nativeProcess = require('process');
 assert.strictEqual(nativeProcess, process,
-  'require("process") should return a reference to global process');
+                   'require("process") should return global process reference');
