@@ -1,9 +1,21 @@
+'use strict';
+
 module.exports = doJSON;
 
 // Take the lexed input, and return a JSON-encoded object
 // A module looks like this: https://gist.github.com/1777387
 
-var marked = require('marked');
+const common = require('./common.js');
+const marked = require('marked');
+
+// customized heading without id attribute
+var renderer = new marked.Renderer();
+renderer.heading = function(text, level) {
+  return '<h' + level + '>' + text + '</h' + level + '>\n';
+};
+marked.setOptions({
+  renderer: renderer
+});
 
 function doJSON(input, filename, cb) {
   var root = {source: filename};
@@ -12,7 +24,7 @@ function doJSON(input, filename, cb) {
   var current = root;
   var state = null;
   var lexed = marked.lexer(input);
-  lexed.forEach(function (tok) {
+  lexed.forEach(function(tok) {
     var type = tok.type;
     var text = tok.text;
 
@@ -31,7 +43,7 @@ function doJSON(input, filename, cb) {
     if (type === 'heading' &&
         !text.trim().match(/^example/i)) {
       if (tok.depth - depth > 1) {
-        return cb(new Error('Inappropriate heading level\n'+
+        return cb(new Error('Inappropriate heading level\n' +
                             JSON.stringify(tok)));
       }
 
@@ -77,7 +89,7 @@ function doJSON(input, filename, cb) {
     //
     // If one of these isnt' found, then anything that comes between
     // here and the next heading should be parsed as the desc.
-    var stability
+    var stability;
     if (state === 'AFTERHEADING') {
       if (type === 'code' &&
           (stability = text.match(/^Stability: ([0-5])(?:\s*-\s*)?(.*)$/))) {
@@ -89,6 +101,8 @@ function doJSON(input, filename, cb) {
         current.list = current.list || [];
         current.list.push(tok);
         current.list.level = 1;
+      } else if (type === 'html' && common.isYAMLBlock(tok.text)) {
+        current.meta = parseYAML(tok.text);
       } else {
         current.desc = current.desc || [];
         if (!Array.isArray(current.desc)) {
@@ -125,7 +139,7 @@ function doJSON(input, filename, cb) {
     finishSection(current, stack[stack.length - 1]);
   }
 
-  return cb(null, root)
+  return cb(null, root);
 }
 
 
@@ -146,7 +160,7 @@ function doJSON(input, filename, cb) {
 //   { type: 'list_item_end' },
 //   { type: 'list_item_start' },
 //   { type: 'text',
-//     text: 'silent: Boolean, whether or not to send output to parent\'s stdio.' },
+//     text: 'silent: Boolean, whether to send output to parent\'s stdio.' },
 //   { type: 'text', text: 'Default: `false`' },
 //   { type: 'space' },
 //   { type: 'list_item_end' },
@@ -168,7 +182,7 @@ function doJSON(input, filename, cb) {
 //          desc: 'string arguments passed to worker.' },
 //        { name: 'silent',
 //          type: 'boolean',
-//          desc: 'whether or not to send output to parent\'s stdio.',
+//          desc: 'whether to send output to parent\'s stdio.',
 //          default: 'false' } ] } ]
 
 function processList(section) {
@@ -181,15 +195,14 @@ function processList(section) {
   list.forEach(function(tok) {
     var type = tok.type;
     if (type === 'space') return;
-    if (type === 'list_item_start') {
+    if (type === 'list_item_start' || type === 'loose_item_start') {
+      var n = {};
       if (!current) {
-        var n = {};
         values.push(n);
         current = n;
       } else {
         current.options = current.options || [];
         stack.push(current);
-        var n = {};
         current.options.push(n);
         current = n;
       }
@@ -231,7 +244,7 @@ function processList(section) {
       // each item is an argument, unless the name is 'return',
       // in which case it's the return value.
       section.signatures = section.signatures || [];
-      var sig = {}
+      var sig = {};
       section.signatures.push(sig);
       sig.params = values.filter(function(v) {
         if (v.name === 'return') {
@@ -248,7 +261,7 @@ function processList(section) {
       // copy the data up to the section.
       var value = values[0] || {};
       delete value.name;
-      section.typeof = value.type;
+      section.typeof = value.type || section.typeof;
       delete value.type;
       Object.keys(value).forEach(function(k) {
         section[k] = value[k];
@@ -259,33 +272,53 @@ function processList(section) {
       // event: each item is an argument.
       section.params = values;
       break;
+
+    default:
+      if (section.list.length > 0) {
+        section.desc = section.desc || [];
+        for (var i = 0; i < section.list.length; i++) {
+          section.desc.push(section.list[i]);
+        }
+      }
   }
 
   // section.listParsed = values;
   delete section.list;
 }
 
+function parseYAML(text) {
+  return common.extractAndParseYAML(text);
+}
 
 // textRaw = "someobject.someMethod(a[, b=100][, c])"
 function parseSignature(text, sig) {
   var params = text.match(paramExpr);
   if (!params) return;
   params = params[1];
-  // the [ is irrelevant. ] indicates optionalness.
-  params = params.replace(/\[/g, '');
-  params = params.split(/,/)
+  params = params.split(/,/);
+  var optionalLevel = 0;
+  var optionalCharDict = {'[': 1, ' ': 0, ']': -1};
   params.forEach(function(p, i, _) {
     p = p.trim();
     if (!p) return;
     var param = sig.params[i];
     var optional = false;
     var def;
-    // [foo] -> optional
-    if (p.charAt(p.length - 1) === ']') {
-      optional = true;
-      p = p.substr(0, p.length - 1);
-      p = p.trim();
+
+    // for grouped optional params such as someMethod(a[, b[, c]])
+    var pos;
+    for (pos = 0; pos < p.length; pos++) {
+      if (optionalCharDict[p[pos]] === undefined) { break; }
+      optionalLevel += optionalCharDict[p[pos]];
     }
+    p = p.substring(pos);
+    optional = (optionalLevel > 0);
+    for (pos = p.length - 1; pos >= 0; pos--) {
+      if (optionalCharDict[p[pos]] === undefined) { break; }
+      optionalLevel += optionalCharDict[p[pos]];
+    }
+    p = p.substring(0, pos + 1);
+
     var eq = p.indexOf('=');
     if (eq !== -1) {
       def = p.substr(eq + 1);
@@ -362,7 +395,7 @@ function parseListItem(item) {
 
 function finishSection(section, parent) {
   if (!section || !parent) {
-    throw new Error('Invalid finishSection call\n'+
+    throw new Error('Invalid finishSection call\n' +
                     JSON.stringify(section) + '\n' +
                     JSON.stringify(parent));
   }
@@ -405,7 +438,7 @@ function finishSection(section, parent) {
   // properties are a bit special.
   // their "type" is the type of object, not "property"
   if (section.properties) {
-    section.properties.forEach(function (p) {
+    section.properties.forEach(function(p) {
       if (p.typeof) p.type = p.typeof;
       else delete p.type;
       delete p.typeof;
@@ -476,14 +509,14 @@ function deepCopy(src, dest) {
 function deepCopy_(src) {
   if (!src) return src;
   if (Array.isArray(src)) {
-    var c = new Array(src.length);
+    const c = new Array(src.length);
     src.forEach(function(v, i) {
       c[i] = deepCopy_(v);
     });
     return c;
   }
   if (typeof src === 'object') {
-    var c = {};
+    const c = {};
     Object.keys(src).forEach(function(k) {
       c[k] = deepCopy_(src[k]);
     });
@@ -502,7 +535,7 @@ var classMethExpr =
   /^class\s*method\s*:?[^\.]+\.([^ \.\(\)]+)\([^\)]*\)\s*?$/i;
 var methExpr =
   /^(?:method:?\s*)?(?:[^\.]+\.)?([^ \.\(\)]+)\([^\)]*\)\s*?$/i;
-var newExpr = /^new ([A-Z][a-z]+)\([^\)]*\)\s*?$/;
+var newExpr = /^new ([A-Z][a-zA-Z]+)\([^\)]*\)\s*?$/;
 var paramExpr = /\((.*)\);?$/;
 
 function newSection(tok) {
