@@ -1,12 +1,7 @@
 #!/usr/bin/env python
 
 import errno
-
-try:
-  import json
-except ImportError:
-  import simplejson as json
-
+import json
 import os
 import re
 import shutil
@@ -155,7 +150,6 @@ def headers(action):
     'config.gypi',
     'src/node.h',
     'src/node_buffer.h',
-    'src/node_internals.h',
     'src/node_object_wrap.h',
     'src/node_version.h',
   ], 'include/node/')
